/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton.org

Copyright (c) 2002, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>

#define kDefaultSnaplen 64
#define kDefaultTimeout 100 // read timeout in milliseconds

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

int gDone = FALSE;
int gTimerDone = FALSE;
int gVerbose = FALSE;
int gContinuous = FALSE;

void HandleSignal(int signum);
void PacketHandler(u_char *userData, const struct pcap_pkthdr *header, const u_char *packet);

// getrusage, gettimeofday, select, getitimer

unsigned long gPacketCount = 0;
unsigned long gByteCount = 0;


#define kSampleSeconds 1

int main(int argc, char **argv)
{
	char *device = NULL;
	int snaplen = kDefaultSnaplen;
	pcap_t *pcapPtr = NULL;
	char ebuf[PCAP_ERRBUF_SIZE]; // error msg buffer
	int timeout = kDefaultTimeout; // read timeout in milliseconds
	int packetsAtOnce = 100;
	int numPackets;
	int ret = 0; // program return
	u_char *userData = NULL; // passed through to PacketHandler
	struct itimerval mytimer;
    struct timeval diff, half;
	unsigned usecs;
	double pps, bps;
	int optRet;

	// parse options
	while( (optRet = getopt( argc, argv, "vc" )) != -1 )
	{
		if( optRet == ':' || optRet == '?' )
			goto usage;
		switch(optRet)
		{
			case 'v':
				gVerbose = TRUE;
				break;
			case 'c':
				gContinuous = TRUE;
				break;
			default:
				goto usage;
		}
	}

	argc -= optind;
	argv += optind;


	if( argc == 1 )
		device = argv[0];
	else if( argc > 1 )
		goto usage;

	if( device && gVerbose )
		printf("Reading from device %s\n",device);

	// read from the packet filter device using libpcap
	// and print the packets in hex on stdout through a callback
	if( !device )
	{
		device = pcap_lookupdev(ebuf);
		if(!device)
		{
			fprintf(stderr,"Could not find a device, %s\n",ebuf);
			ret = 1;
			goto cleanup;
		}
		printf("Defaulting to device %s\n",device);
	}

	
	pcapPtr = pcap_open_live(device, snaplen, TRUE, timeout, ebuf);
	if( !pcapPtr )
	{
		fprintf(stderr,"Unable to open %s for reading, %s\n",device,ebuf);
		ret = 1;
		goto cleanup;
	}

	(void)signal(SIGABRT,HandleSignal);
	(void)signal(SIGTERM,HandleSignal);
	(void)signal(SIGQUIT,HandleSignal);
	(void)signal(SIGINT,HandleSignal);
	(void)signal(SIGALRM,HandleSignal);


	// set a timer and loop until it's done.

	while(!gDone)
	{
		// reset counters
		gPacketCount = 0;
		gByteCount = 0; 
		
		// reset the timer
		mytimer.it_interval.tv_sec = kSampleSeconds * 2;
		mytimer.it_interval.tv_usec = 0;
		mytimer.it_value.tv_sec = kSampleSeconds * 2;
		mytimer.it_value.tv_usec = 0;

		half.tv_sec = kSampleSeconds;
		half.tv_usec = 0;

		setitimer(ITIMER_REAL, &mytimer,NULL);
// #define PrintTV(lbl,tv) printf("%s %ld sec, %ld usec\n",lbl,(tv).tv_sec,(tv).tv_usec) 
#define PrintTV(lbl,tv)

		for(;;)
		{
			numPackets = pcap_dispatch(pcapPtr, packetsAtOnce,PacketHandler, userData);

			getitimer(ITIMER_REAL,&mytimer);
			PrintTV("it_interval",mytimer.it_interval);
			PrintTV("it_value",mytimer.it_value);
			timersub(&(mytimer.it_interval),&(mytimer.it_value),&diff);
			PrintTV("diff",diff);

			if( gDone || gTimerDone || timercmp(&diff,&half,>=) )
				break; // our timer has elapsed.
		}
		// diff is amount of time that has passed 
		// so figure it out in microseconds
		usecs = diff.tv_sec * 1000000;
		usecs += diff.tv_usec;

		// now figure out packets per second and bytes per second
		pps = (double)gPacketCount;
		pps /= (double)usecs / (double)1000000;

		bps = (double)gByteCount;
		bps /= (double)usecs / (double)1000000;

		if( gVerbose )
		{
			printf("%f seconds, %lu packets, %lu bytes\n",
				(double)usecs/1000000.0, gPacketCount, gByteCount);
		}
		printf("%f packets per second, %f bytes per second\n",pps,bps);

		if( ! gContinuous )
			break; // just run once.
	}
	



	// clear the timer out
	timerclear(&mytimer.it_interval);
	timerclear(&mytimer.it_value);
	setitimer(ITIMER_REAL, &mytimer,NULL);

cleanup:
	if( pcapPtr )
		pcap_close(pcapPtr);

	return ret;
usage:
	fprintf(stderr,"Usage: pps [-vc] [devicename]\n");
	return 2;
}



void PacketHandler(u_char *userData, const struct pcap_pkthdr *header, const u_char *packet)
{
	// print out the packet in hex, 16 bytes (32 hex chars) per line with
	// a space after every fourth hex digit.

	gPacketCount++;
	gByteCount += header->len;

	/*
	int len = header->caplen; // length of data actually available to us
	int lines;
	char line[ 42 ]; // 32 digits, 8 spaces, 1 newline, and 1 0 byte
	const u_char *bptr = packet;
	char *lptr;
	int cnt,index;
	char kHexTable[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

	// round up
//	printf("--- len = %d\n",len);

//	lines = (len+15) / 16;
	lines = (len+15) >> 4;


	while(lines--)
	{
		lptr = line;
		cnt = len > 16 ? 16 : len;
		len -= cnt;

		for(index=0;index<cnt;index++)
		{
			*lptr++ = kHexTable[ *bptr >> 4 ];
			*lptr++ = kHexTable[ *bptr++ & 0xF ];
			// if( index % 2 == 1 )
			if( (index&1) == 1 )
				*lptr++ = ' ';

		}
		*lptr++ = '\n';


// this write is like 99.99999% of the CPU of this program
// need to just buffer it up and output it when the buffer is full.
		write(1,line,lptr-line);
	}
	*/

}

void HandleSignal(int signum)
{
	switch(signum)
	{
		case SIGALRM:
			gTimerDone = TRUE;
				break;
		case SIGINT:
		case SIGQUIT:
		case SIGABRT:
		case SIGTERM:
			gDone = TRUE;
			break;
		default:
	}
}

